@@ -1,9 +1,36 @@
+#############################################################################
+#   Copyright (c) 2018 Giorgio A. Spedicato
+#
+#   This program is free software; you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation; either version 2 of the License, or
+#   (at your option) any later version.
+#
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#
+#   You should have received a copy of the GNU General Public License
+#   along with this program; if not, write to the
+#   Free Software Foundation, Inc.,
+#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
+#
+#############################################################################
+###
+###         random generator functions
+###
+
+
 # TODO: function to generate Iaxn variates
 # TODO: check axn
 # 
 # Author: Giorgio A. Spedicato
 ###############################################################################
 
+
+
+
 ##########random variables Tx and Kx generators 
 
 #
@@ -24,11 +51,40 @@ out=ifelse((x>getOmega(object)),0,dxt(object = object,
 
 
 
+#' @name rLifes
+#' @aliases rLifexyz
+#' @title Function to generate random future lifetimes
+#'
+#' @param n Number of variates to generate
+#' @param object An object of class lifetable
+#' @param x The attained age of subject x, default value is 0
+#' @param k Number of periods within the year when it is possible death to happen, default value is 1
+#' @param type Either \code{"Tx"} for continuous future lifetime, \code{"Kx"} for curtate furture lifetime (can be abbreviated).
+#' 
+#' @details Following relation holds for the future life time: \eqn{T_x=K_x+0.5}
+#' @references 	Actuarial Mathematics (Second Edition), 1997, by Bowers, N.L., Gerber, H.U., Hickman, J.C., Jones, D.A. and Nesbitt, C.J.
+#' @note The function is provided as is, without any warranty regarding the accuracy of calculations. The author disclaims any liability for eventual 	losses arising from direct or indirect use of this software.
+#' @seealso \code{\linkS4class{lifetable}}, \code{\link{exn}}
+#'
+#' @return A numeric vector of n elements.
+#'
+#' @examples
+#' \dontrun{
+#' ##get 20000 random future lifetimes for the Soa life table at birth
+#' data(soa08Act)
+#' lifes=rLife(n=20000,object=soa08Act, x=0, type="Tx")
+#' check if the expected life at birth derived from the life table is statistically equal 
+#' to the expected value of the sample
+#' t.test(x=lifes, mu=exn(soa08Act, x=0, type="continuous"))
+#' }
+#' @export
 rLife<-function(n,object, x=0,k=1, type="Tx")
 {
 	if(missing(n)) stop("Error! Needing the n number of variates to return")
 	if((class(object) %in% c("lifetable", "actuarialtable"))==FALSE) stop("Error! Objectx needs be lifetable or actuarialtable objects")
 	if(x>getOmega(object)) stop("Error! x > maximum attainable age")
+  type <- testtypelifearg(type)
+  
 	out=numeric(n)
 	const2Add=0
 	if(type=="Tx") const2Add=0.5/k # the continuous future lifetime 
@@ -55,23 +111,42 @@ rLife<-function(n,object, x=0,k=1, type="Tx")
 #t.test(x=ciao, mu=exn(soa08Act,25))
 
 
-rLifexyz=function(n,tablesList,x,k=1, type="Tx")
+#' @rdname rLifes
+#'
+#' @param tablesList An list of lifetables
+#' @examples
+#' \dontrun{
+#' #assessment of curtate expectation of future lifetime of the joint-life status
+#' #generate a sample of lifes
+#' data(soaLt)
+#' soa08Act=with(soaLt, new("actuarialtable",interest=0.06,x=x,lx=Ix,name="SOA2008"))
+#' tables=list(males=soa08Act, females=soa08Act)
+#' xVec=c(60,65)
+#' test=rLifexyz(n=50000, tablesList = tables,x=xVec,type="Kx")
+#' #check first survival status
+#' t.test(x=apply(test,1,"min"),mu=exyzt(tablesList=tables, x=xVec,status="joint"))
+#' #check last survival status
+#' t.test(x=apply(test,1,"max"),mu=exyzt(tablesList=tables, x=xVec,status="last"))
+#' }
+#' @export
+rLifexyz <- function(n,tablesList,x,k=1, type="Tx")
 {
-	
 	#initial checkings
 	numTables=length(tablesList)
 	if(length(x)!=numTables) stop("Error! Initial ages vector length does not match with number of lives")
 	for(i in 1:numTables) {
 		if(!(class(tablesList[[i]]) %in% c("lifetable", "actuarialtable"))) stop("Error! A list of lifetable objects is required")
 	}	
-	outVec=numeric(0)
+	type <- testtypelifearg(type)
+	
+	outVec <- numeric(0)
 	for(i in 1:numTables)
 	{
-		outI=numeric(n)
-		outI=rLife(n=n,object=tablesList[[i]],x=x[i],k=k,type=type)
-		outVec=c(outVec,outI)
+		outI <- numeric(n)
+		outI <- rLife(n=n,object=tablesList[[i]],x=x[i],k=k,type=type)
+		outVec <- c(outVec,outI)
 	}	
-	out=matrix(data=outVec,nrow=n,ncol=numTables,byrow=FALSE)
+	out<- matrix(data=outVec,nrow=n,ncol=numTables,byrow=FALSE)
 	return(out)
 }
 
@@ -152,6 +227,7 @@ rLifexyz=function(n,tablesList,x,k=1, type="Tx")
 #seems to work
 .fAxyzn<-function(T,y,n, i, m, k, status)
 {
+  status <- teststatusarg(status)
 	out=numeric(1)
 	temp=T-y
 	T=ifelse(status=="joint",min(temp),max(temp)) #redefines T
@@ -202,15 +278,18 @@ rLifexyz=function(n,tablesList,x,k=1, type="Tx")
 #APV
 #
 
+
+
 .faxn<-function(T,y,n, i, m, k=1, payment="advance")
 {
 	out=numeric(1)
+	payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
 	K=T-y #number of years to live
 		if(K<m) { #if policyholder dies before inception of the annuity
 			out=0 #no payment is due
 		} else {
 		  times=seq(from=m, to=min(m+n-1/k,K),by=1/k) #else it pays from m to the min of m + n - 1/k
-      if (payment=="arrears") times = times + 1/k;
+      if (payment=="immediate") times = times + 1/k;
  		  out=presentValue(cashFlows=rep(1/k, length(times)), timeIds=times, interestRates=i)
 		}
 	return(out)
@@ -238,13 +317,16 @@ rLifexyz=function(n,tablesList,x,k=1, type="Tx")
 .faxyzn<-function(T,y,n, i, m, k=1,status, payment="advance")
 {
 	out=numeric(1)
+	payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+	status <- teststatusarg(status)
+	
 	temp=T-y
 	K=ifelse(status=="joint",min(temp),max(temp))
 	if(K<m) { #if policyholder dies before inception of the annuity
 				out=0 #no payment is due
 			} else {
 				times=seq(from=m, to=min(m+n-1/k,K),by=1/k) #else it pays from m to the min of m + n - 1/k
-				if (payment=="arrears") times = times + 1/k #copy from univariate
+				if (payment=="immediate") times = times + 1/k #copy from univariate
 				out=presentValue(cashFlows=rep(1/k, length(times)), timeIds=times, interestRates=i)
 			}	
 	return(out)
@@ -280,10 +362,47 @@ rLifexyz=function(n,tablesList,x,k=1, type="Tx")
 #m=0
 #k=1
 
-
+#' @name rLifeContingencies
+#' @aliases rLifeContingenciesXyz
+#' @title Function to generate samples from the life contingencies stochastic variables
+#'
+#' @param n Size of sample
+#' @param lifecontingency A character string, either \code{"Exn"}, \code{"Axn"}, 
+#' \code{"axn"}, \code{"IAxn"} or \code{"DAxn"}
+#' @param object An \code{actuarialtable} object.
+#' @param x Policyholder's age at issue time;  for \code{rLifeContingenciesXyz} a numeric vector of 
+#' the same length of \code{object}, containing the policyholders' ages
+#' @param t The lenght of the insurance. Must be specified according to the 
+#' present value of benefits definition.
+#' @param i The interest rate, whose default value is the \code{actuarialtable} interest rate slot value.
+#' @param m Deferring period, default value is zero.
+#' @param k Fractional payment, default value is 1.
+#' @param parallel Uses the parallel computation facility.
+#' @param payment The Payment type, either \code{"advance"} for the annuity due (default) 
+#' or \code{"arrears"} for the annuity immediate. 
+#' Alternatively, one can use \code{"due"} or \code{"immediate"} 
+#' respectively (can be abbreviated).
+#'
+#' @return A numeric vector
+#' @export
+#'
+#' @examples
+#' \dontrun{
+#' 	#assumes SOA example life table to be load
+#' 	data(soaLt)
+#' 	soa08Act=with(soaLt, new("actuarialtable",interest=0.06, x=x,lx=Ix,name="SOA2008"))
+#' 	out<-rLifeContingencies(n=1000, lifecontingency="Axn",object=soa08Act, x=40,
+#' 	t=getOmega(soa08Act)-40, m=0)
+#' 	APV=Axn(soa08Act,x=40)
+#' 	#check if out distribution is unbiased
+#' 	t.test(x=out, mu=APV)$p.value>0.05
+#' }
 rLifeContingencies<-function (n, lifecontingency, object, x, t, i = object@interest, 
 		m = 0, k = 1, parallel = FALSE, payment="advance") 
 {
+  payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+  lifecontingency <- testlifecontarg(lifecontingency)
+  
 	deathsTimeX = numeric(n)
 	outs = numeric(n)
 	if (k == 1) 
@@ -384,9 +503,32 @@ rLifeContingencies<-function (n, lifecontingency, object, x, t, i = object@inter
 #APV
 #mean(ciao)
 
+#' @rdname rLifeContingencies
+#' @param tablesList A list of \code{actuarialtable} objects
+#' @param status Either \code{"joint"} for the joint-life status model or \code{"last"} 
+#' for the last-survivor status model (can be abbreviated).
+#' @examples 
+#' \dontrun{
+#' data(soa08Act)
+#' n=10000
+#' lifecontingency="Axyz"
+#' tablesList=list(soa08Act,soa08Act)
+#' x=c(60,60); i=0.06; m=0; status="joint"; t=30; k=1
+#' APV=Axyzn(tablesList=tablesList,x=x,n=t,m=m,k=k,status=status,type="EV")
+#' samples<-rLifeContingenciesXyz(n=n,lifecontingency = lifecontingency,tablesList = tablesList,
+#' x=x,t=t,m=m,k=k,status=status, parallel=FALSE)
+#' APV
+#' mean(samples)
+#' } 
+#' @export
+
 rLifeContingenciesXyz<-function(n,lifecontingency, tablesList, x,t,i, 
 		m=0,k=1, status="joint", parallel=FALSE, payment="advance")
 {
+  payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+  lifecontingency <- testlifecontarg2(lifecontingency)
+  status <- teststatusarg(status)
+  
 	numTables=length(tablesList)
 	#gets the missing i
 	if(missing(i)) {
@@ -407,7 +549,8 @@ rLifeContingenciesXyz<-function(n,lifecontingency, tablesList, x,t,i,
 	temp=matrix(nrow=n, ncol=numTables)
 	outs=numeric(n)
 	#fractional payment are handled using countinuous lifetime simulation
-	if(k==1) temp=x+rLifexyz(n=n,tablesList=tablesList,x=x, k=k,type="Kx") else temp=x+rLifexyz(n=n,tablesList=tablesList,x=x,k=k,type="Tx") #this to handle fractional payments (assume continuous...)
+	if(k==1) temp=x+rLifexyz(n=n,tablesList=tablesList,x=x, k=k,type="Kx") 
+	else temp=x+rLifexyz(n=n,tablesList=tablesList,x=x,k=k,type="Tx") #this to handle fractional payments (assume continuous...)
 
 	deathsTimeX<-temp	
 	if(parallel==TRUE) {
@@ -447,6 +590,8 @@ rLifeContingenciesXyz<-function(n,lifecontingency, tablesList, x,t,i,
 getLifecontingencyPv<-function (deathsTimeX, lifecontingency, object, x, t, i = object@interest, 
 		m = 0, k = 1,  payment="advance") 
 {
+  payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+  
 	outs = numeric(length(deathsTimeX))
 	if (lifecontingency == "Axn") 
 		outs = sapply(deathsTimeX, .fAxn, y = x, n = t, i = i, 
@@ -475,8 +620,11 @@ getLifecontingencyPv<-function (deathsTimeX, lifecontingency, object, x, t, i =
 
 
 getLifecontingencyPvXyz<-function(deathsTimeXyz,lifecontingency, tablesList, x,t,i, 
-		m=0,k=1, status="joint",  payment="advance")
+		m=0,k=1, status="joint", payment="advance")
 {
+  payment <- testpaymentarg(payment) # "advance"->"due"; "arrears"->"immediate"
+  status <- teststatusarg(status)
+  
 	numTables=length(tablesList)
 	if(ncol(deathsTimeXyz)!=numTables) stop("Error! deathTimeXyz columns should match the number of life tables!")
 	#gets the missing i
